// Shows the effect of the flags WinStrXYWrap supports.
// WinStrXYWrap and DrawStrWidthP are reimplemented in this
// program, that's why it can work on any AMS version,
// including very old AMS 1.00 for 92+.

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define SAVE_SCREEN           // Save/Restore LCD Contents
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#define NO_CALC_DETECT
#define MIN_AMS 100           // Compile for AMS 1.00 or higher
#define NO_AMS_CHECK

#include <tigcclib.h>         // Include All Header Files

// Reimplemented AMS 2.00+ function DrawStrWidthP. Unlike the AMS function, this 
// function receives its parameters in the registers...
unsigned short DrawStrWidthP(register const char *str asm("a0"), register short len asm("d0"), register short Font asm("d2"));
asm("DrawStrWidthP:
    move.w   %d0,%d1
    beq.s    __end_DrawStrWidthP__
    btst     #0,%d0
    beq.s    __allocate_on_the_stack_DSWP__
    addq.w   #1,%d1
__allocate_on_the_stack_DSWP__:
    suba.w   %d1,%sp
    
    movea.l  %sp,%a1
    subq.w   #1,%d0
__loop_copy_DrawStrWidthP__:
    move.b   (%a0)+,(%a1)+
    dbf      %d0,__loop_copy_DrawStrWidthP__
    clr.b    (%a1)
    
    addq.w   #8,%d1
    move.w   %d1,-(%sp) | save d1 (DrawStrWidth can destroy it)...
    move.w   %d2,-(%sp)
    pea      4(%sp)
    movea.l  0xC8.w,%a0
    movea.l  0x197*4(%a0),%a0 | DrawStrWidth
    jsr      (%a0)
    
    adda.w   6(%sp),%sp
__end_DrawStrWidthP__:
    rts
");

short __attribute__((__stkparm__)) WinStrXYWrap(WINDOW *w, WIN_COORDS x, WIN_COORDS y, char *s, short flags)
{
  char *bs, c;
  short numChars, pixWid, startX, SysFontY;
  startX = (flags & WWF_WRAP_BACK_TO_ZERO)? 0: x;
  SysFontY = ((w->CurFont == F_4x6) ? 6 : (w->CurFont == F_6x8 ? 8 : 10));
  do {
    bs = s;
    numChars = 0;
    while ((c = *bs) != 0) {
      if (c == 0x0A) break;
      if (c == ' ') if (numChars) break;
      numChars++;
      if ((flags & WWF_WRAP_ON_COMMAS) && c == ',') break;
      bs++;
    }
    if (numChars) {
      pixWid = DrawStrWidthP(s,numChars,w->CurFont);
      if (x + pixWid >= WinWidth(w)) {
        x = startX;
        y += SysFontY;
        if (*s == ' ') {
          if (numChars) numChars--;
          s++;
        }
      }
      while (numChars--) {
        c = *s++;
        if (flags & WWF_DRAW) WinCharXY(w,x,y,c,1);
        x += FontCharWidth(c);
      }
    }
    else {
      x = startX;
      y += SysFontY;
      s++;
    }
  } while (*bs);
  return y;
}

void _main(void)
{
  WINDOW w;
  
  WinOpen(&w,&(WIN_RECT){10,10,130,90},WF_SAVE_SCR);
  ClrScr();
  DrawClipRect(&(WIN_RECT){10,10,130,90},&(SCR_RECT){{0,0,239,127}},A_NORMAL|B_NORMAL);
  WinStrXYWrap(&w,10,0,"Trying a string,trying\x0A...",WWF_DRAW|WWF_WRAP_ON_COMMAS|WWF_WRAP_BACK_TO_ZERO);
  
  GKeyIn(NULL,0);
  ScrRectFill(&(SCR_RECT){{11,11,129,89}},&(SCR_RECT){{0,0,239,127}},A_REVERSE);
  WinStrXYWrap(&w,10,0,"Trying a string,trying\x0A...",WWF_DRAW|WWF_WRAP_ON_COMMAS);
  
  GKeyIn(NULL,0);
  ScrRectFill(&(SCR_RECT){{11,11,129,89}},&(SCR_RECT){{0,0,239,127}},A_REVERSE);
  WinStrXYWrap(&w,10,0,"Trying a string,trying\x0A...",WWF_DRAW);
  
  GKeyIn(NULL,0);
  WinClose(&w);
}
