| This routine returns the HANDLE of the block containing the address
| where the given pointer points to, H_NULL in case of failure.
|
| This function is originally known from kernels. It can be useful sometimes,
| there are interesting things in kernels, as I've said multiple times...
| My opinion is that it's a shame AMS doesn't have such a function, and that
| the kernels that provide that function are kernels (so, their functions
| can't be used in AMS native mode, so-called _nostub mode)...
|
| Copyright  Lionel Debroux 2003, designed for TIGCCLIB.
|
| The C code that does the same thing (but less optimized) would look like:
/*
__attribute__((__regparm__)) HANDLE Ptr2Hd_C(void *ptr)
{
  HANDLE h = H_NULL;
  void *ptr2;
  while (h < 2000)
  {
     h++;
     ptr2 = HeapDeref(h);
     if (ptr >= ptr2 && (unsigned char *)ptr < (unsigned char *)ptr2 + HeapSize(h)) return h;
  }
  return H_NULL;
}
*/

.data
.even
.xdef Ptr2Hd
Ptr2Hd:
| Input: a0.l: pointer (__attribute__((__regparm__)) mode).
| Output: d0.w: handle.
| Destroyed: d0, d1, d2, a0, a1.

| Save the registers that mustn't be destroyed...
        movem.l  %d3-%d4/%a2-%a4,-(%sp)

| Save the pointer into a2 which can't be destroyed by HeapDeref and HeapSize.
        move.l   %a0,%a2
| Put the addresses of HeapDeref and HeapSize in the registers.
        movea.l  0xC8.w,%a4
        movea.l  0x96*4(%a4),%a3 | HeapDeref.
        movea.l  0x9E*4(%a4),%a4 | HeapSize.
        
| There are 2000 handles, and HeapDeref(H_NULL); and HeapSize(H_NULL);
| mustn't be done (HeapSize(H_NULL); causes a crash).
        move.w   #2000-1,%d3

| Push the argument passed to HeapDeref and HeapSize onto the stack just once,
| it will be more efficient than pushing it and adjusting sp each time.
| However, this is dirty, as the function may destroy its own arguments
| (for example, sprintf does) and may break the trick. HeapDeref and HeapSize
| don't, but this piece of code is definitely not good programming practice,
| since it may fail with several functions.
        clr.w    -(%sp)

| Never dereference H_NULL...
__Ptr2Hd_loop__:
        addq.w   #1,(%sp)
        subq.w   #1,%d3
| Are we looking past the end of the heap table ?
        blt.s    __Ptr2Hd_not_found__
| No, continue.
        jsr      (%a3) | HeapDeref.
| Most efficient way to test a pointer against NULL.
        move.l   %a0,%d4
        beq.s    __Ptr2Hd_loop__ | NULL -> next handle.

| If a2 is lower than the lowest address of that handle, there's no point in
| making more tests with that handle.
        cmpa.l   %d4,%a2
        bls.s    __Ptr2Hd_loop__

        jsr      (%a4) | HeapSize.

| Add the value returned by HeapSize to the lowest address of that handle.
        add.l    %d0,%d4

| Is a2 within that handle ?
        cmp.l    %a2,%d4
        bls.s    __Ptr2Hd_loop__

| Yes, we've finished.
        move.w   (%sp)+,%d0
        bra.s    __Ptr2Hd_end__

__Ptr2Hd_not_found__:
| We've not found any handle. Return H_NULL to show failure.
        addq.l   #2,%sp
        clr.w    %d0


| Cleanup and return...
__Ptr2Hd_end__:
        movem.l  (%sp)+,%d3-%d4/%a2-%a4
        rts

.even
